/*-
 * Copyright (c) 2008  Marin Atanasov Nikolov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <dirent.h>
# include "fnode.h"

/* saves a matched pattern into a stack */
void fnode_push (FNODEPUSHARGS)
{
   fnode_t *newnode = (fnode_t *) malloc (sizeof(fnode_t));
   newnode->fname = (char *) malloc (strlen(fname) + 1);
   newnode->fpath = (char *) malloc (strlen(fpath) + 1);
   
   newnode->desc = (desc == NULL) ? NULL : (char *) malloc (strlen(desc) + 1);
   newnode->maint = (maint == NULL) ? NULL : (char *) malloc (strlen(maint) + 1);
   
   if (!newnode || !newnode->fname || !newnode->fpath) {
      perror ("==>> malloc");
      exit (1);
   }
   
   strcpy (newnode->fname, fname);
   strcpy (newnode->fpath, fpath);
   if (desc)
      strcpy (newnode->desc, desc);
   if (maint)
      strcpy (newnode->maint, maint);
   
   newnode->next = *head;
   *head = newnode;
}

/* displays all matched pattern entries */
void fnode_display (FNODEDISPLAYARGS)
{
   char buf[MAXNAMLEN + 1];
   int i = 0;
   fnode_t *current = head;
   
   fprintf (stdout, "==>> Total packages found: %d\n", numfiles);
   fprintf (stdout, "==>> List of packages matching the given pattern.. \n\n%s", LONGLINE);
   for ( ; i < numfiles; i++, current = current->next) {
      fprintf (stdout, "(%.3d)\t%-30s (in %-s)\n", i + 1, current->fname, current->fpath);
      if (current->desc)
         fprintf (stdout, "\tDescription: %s\n", current->desc);
      if (current->maint)
         fprintf (stdout, "\tMaintained by: %s\n\n", current->maint);
   }
   
   fprintf (stdout, "%s\n==>> Total packages found: %d\n", LONGLINE, numfiles);
}

/* asks the user to choose a package */
int fnode_choose_pkg (FNODECHOOSEPKGARGS)
{
   int choice;
   char buf[MAXNAMLEN + 1];
   
   do {
      fprintf (stdout, "==>> Choose the package number you wish to install [1-%d]: ", numfiles);
      fgets (buf, 10, stdin);
      choice = atoi (buf);
   } while (choice < 1 || choice > numfiles);
   
   return choice;
}

/* locates a package node using an index number */
void fnode_locate (FNODELOCATEARGS)
{
   int i = 1;
   fnode_t *current = head;
   
   for ( ; i < index; i++, current = current->next) ; 
   
   *pkgfile = current;
}

/* deallocates everything */
void fnode_shutdown (FNODESHUTDOWNARGS)
{
   fnode_t *current = head, *next = NULL;
      
   while (current) {
      next = current->next;
      free (current);
      current = next;
   }   
}
